/*
 * ////////////////////////////////////////////////////////////////////////////
 * // Copyright (c) 2010, Rob Eden All Rights Reserved.
 * //
 * // This library is free software; you can redistribute it and/or
 * // modify it under the terms of the GNU Lesser General Public
 * // License as published by the Free Software Foundation; either
 * // version 2.1 of the License, or (at your option) any later version.
 * //
 * // This library is distributed in the hope that it will be useful,
 * // but WITHOUT ANY WARRANTY; without even the implied warranty of
 * // MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * // GNU General Public License for more details.
 * //
 * // You should have received a copy of the GNU Lesser General Public
 * // License along with this program; if not, write to the Free Software
 * // Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
 * ///////////////////////////////////////////////////////////////////////////////
 */

package gnu.trove.impl.hash;

import junit.framework.TestCase;


/**
 * Static functions for unit tests
 */
public class HashTestKit {
	/**
	 * Confirm that the internal FREE counter matches the values in the slots.
	 */
	public static void checkFreeSlotCount( THash hash, Object[] slot_keys,
		Object free_marker ) {

		int free_counter = hash._free;

		int count = 0;
		for( Object slot_key : slot_keys ) {
			if ( slot_key == free_marker ) count++;
		}

		TestCase.assertEquals( free_counter, count );
	}
}
